import piapi
from PyQt6.QtCore import QRunnable, pyqtSlot, pyqtSignal, QObject, QCoreApplication
import time
import logging
import pi_pb2 as pipb

class PIAPISignals(QObject):
    messageReceived = pyqtSignal(object)
    detectionMeasurementReceived = pyqtSignal(object)
    referenceMeasurementReceived = pyqtSignal(object)
    boardIdentificationReceived = pyqtSignal(object)
    samplingConfigurationReceived = pyqtSignal(object)
    thresholdConfigurationReceived = pyqtSignal(object)
    thresholdStatusReportReceived = pyqtSignal(object)
    systemStatusReceived = pyqtSignal(object)

class PIAPIWorker(QRunnable):
    api: piapi.PIAPI
    should_stop: bool
    log: logging.Logger
    signals: PIAPISignals

    def __init__(self, api: piapi.PIAPI):
        super().__init__()
        self.api = api
        self.should_stop = False
        self.log = logging.getLogger("apiworker")
        self.signals = PIAPISignals()

    def stop(self):
        self.should_stop = True

    @pyqtSlot()
    def run(self):
        """API handling loop which emits self.signals.messageReceived for received messages"""
        self.log.info("starting loop")

        while not self.should_stop:
            msgs = self.api.receive_messages(timeout=0.01, abort_early=True)
            for m in msgs:
                self.signals.messageReceived.emit(m)
                if type(m) == pipb.DetectionMeasurement:
                    self.log.log(logging.TRACE, "emitting measurement %d" % m.timestamp_us)
                    self.signals.detectionMeasurementReceived.emit(m)
                elif type(m) == pipb.BoardIdentification:
                    self.signals.boardIdentificationReceived.emit(m)
                elif type(m) == pipb.ReferenceMeasurement:
                    self.signals.referenceMeasurementReceived.emit(m)
                elif type(m) == pipb.SamplingConfiguration:
                    self.signals.samplingConfigurationReceived.emit(m)
                elif type(m) == pipb.ThresholdConfiguration:
                    self.signals.thresholdConfigurationReceived.emit(m)
                elif type(m) == pipb.ThresholdStatusReport:
                    self.signals.thresholdStatusReportReceived.emit(m)
                elif type(m) == pipb.SystemStatus:
                    self.signals.systemStatusReceived.emit(m)

            time.sleep(0.001)
