import logging
import piapi
import sys
import numpy as np
import pi_pb2
import time
import datetime
logging.TRACE = 9
logging.basicConfig(level=logging.DEBUG, format="%(asctime)s %(levelname)s: %(message)s")

def main():
    port = sys.argv[1]
    api = piapi.PIAPI(port=port)
    start_dt = datetime.datetime.now()
    next_status_print_time = time.time() + 1
    raw_sample_msg_received = 0
    data_ts = []
    data_samples = []
    print("Acquiring RawSample messages, press Ctrl-C to stop and save")
    try:
        while True:
            msgs = api.receive_messages(timeout=0.1, abort_early=False)
            for m in msgs:
                if type(m) == pi_pb2.RawSample:
                    data_ts.append(m.timestamp_us)
                    data_samples.append(m.adc_sample)
                    raw_sample_msg_received += 1
            if time.time() < next_status_print_time:
                print("RawSample messages received: %d" % raw_sample_msg_received)
                next_status_print_time = time.time() + 1
    except KeyboardInterrupt:
        if len(data_ts) > 0:
            ts_np = np.array(data_ts)
            samples_np = np.array(data_samples)
            filename = "raw_samples_%s.npz" % start_dt.strftime("%Y_%m_%d_%H_%M_%S")
            np.savez(filename, timestamp=ts_np, sample=samples_np)
            print("%d samples saved to %s" % (raw_sample_msg_received, filename))
        else:
            print("No samples received, not saving anything")

if __name__ == "__main__":
    main()

